#include <ruby.h>
#include <node.h>
#include "yarvcore.h"

#ifndef _YARV_H_INCLUDED_
#define _YARV_H_INCLUDED_


VALUE yarv_yield _((VALUE val));

/* original API */

#if YARVEXT
RUBY_EXTERN int yarvIsWorking;
#define IS_YARV_WORKING() (yarvIsWorking)
#define SET_YARV_START()  (yarvIsWorking = 1)
#define SET_YARV_STOP()   (yarvIsWorking = 0)
#else
#define IS_YARV_WORKING() 1
#define SET_YARV_START()
#define SET_YARV_STOP() 
#endif


#if YARV_THREAD_MODEL == 2
extern yarv_thread_t *yarvCurrentThread;
extern yarv_vm_t *theYarvVM;

static inline VALUE
yarv_get_current_running_vm_value()
{
  return theYarvVM->self;
}

static inline VALUE
yarv_get_current_running_thread_value()
{
  return yarvCurrentThread->self;
}

static inline yarv_vm_t *
yarv_get_current_running_vm()
{
  return theYarvVM;
}

static inline yarv_thread_t *
yarv_get_current_running_thread()
{
  return yarvCurrentThread;
}

static inline void
yarv_set_current_running_thread(yarv_thread_t *th)
{
  yarvCurrentThread = th;
}

#else
#error "unsupported thread model"
#endif

#define GET_THREAD() yarv_get_current_running_thread()
#define GET_VM()     yarv_get_current_running_vm()

struct yarv_yield_data{
  yarv_thread_t *th;
  yarv_block_t *block;
  VALUE *pc;
  VALUE *sp;
  VALUE spbuff;
  int argc;
  int local_size;
};

VALUE thread_yield_light_prepare _((VALUE self, int argc, VALUE *argv,
                                    struct yarv_yield_data *data));

VALUE th_invoke_yield(yarv_thread_t *th, int argc, VALUE *argv);

VALUE th_call0(yarv_thread_t *th, VALUE klass, VALUE recv,
               VALUE id, ID oid, int argc, const VALUE *argv,
               NODE *body, int nosuper);

VALUE *yarv_svar(int);

VALUE th_call_super(yarv_thread_t *th, int argc, const VALUE *argv);

VALUE yarv_backtrace(int lev);

VALUE yarvcore_eval_parsed(VALUE node, VALUE file);

VALUE th_invoke_proc(yarv_thread_t *th, yarv_proc_t *proc, int argc, VALUE *argv);
VALUE th_make_proc(yarv_thread_t *th, yarv_control_frame_t *cfp, yarv_block_t *block);
VALUE th_make_env_object(yarv_thread_t *th, yarv_control_frame_t *cfp);
VALUE yarvcore_eval(VALUE self, VALUE str, VALUE file, VALUE line);

int yarv_block_given_p();

#endif
